
extern crate proc_macro;

use proc_macro::{TokenStream, TokenTree};
use quote::quote;
use syn::{parse_macro_input, parse_quote, Expr, ItemStruct};

#[proc_macro_attribute]
pub fn advent_day(attrs: TokenStream, item: TokenStream, ) -> TokenStream {
    let input = parse_macro_input!(item as ItemStruct);
    let ident = &input.ident;

    let mut parts = vec![];
    for token in attrs.into_iter() {
        match token {
            TokenTree::Ident(ident) => parts.push(ident.to_string()),
            _ => continue
        }
    }

    let call1 = format!("{}::new()", parts[0]);
    let call2 = format!("{}::new()", parts[1]);
    let part1: Expr = parse_quote!(#call1);
    let part2: Expr = parse_quote!(#call2);
    
    println!("{:?} {:?}", part1, part2);
    
    let output = quote! {
        pub struct #ident {
            part1: Box::<dyn AdventDayPart1>,
            part2: Box::<dyn AdventDayPart2>
        }

        impl Day4 {
            pub fn new() -> Self {
                Day4 {
                    part1: Box::new(#part1),
                    part2: Box::new(#part2)
                }
            }
        }          

        impl AdventDay for #ident {
            fn puzzle1(&mut self) -> Result<u64, AdventError> {
                self.part1.read_input()?;
                self.part1.solve()
            }

            fn puzzle2(&mut self) -> Result<u64, AdventError> {
                self.part2.read_input()?;
                self.part2.solve()
            }
        }
    };
    output.into()

}
